@@ -1,41 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2017 Ford Motor Company
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtRemoteObjects module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2017 Ford Motor Company
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qremoteobjectabstractitemmodelreplica.h"
 #include "qremoteobjectabstractitemmodelreplica_p.h"
@@ -48,6 +13,15 @@
 
 QT_BEGIN_NAMESPACE
 
+QT_IMPL_METATYPE_EXTERN_TAGGED(QtPrivate::ModelIndex, QtPrivate__ModelIndex)
+QT_IMPL_METATYPE_EXTERN_TAGGED(QtPrivate::IndexList, QtPrivate__IndexList)
+QT_IMPL_METATYPE_EXTERN_TAGGED(QtPrivate::DataEntries, QtPrivate__DataEntries)
+QT_IMPL_METATYPE_EXTERN_TAGGED(QtPrivate::MetaAndDataEntries, QtPrivate__MetaAndDataEntries)
+QT_IMPL_METATYPE_EXTERN_TAGGED(QtPrivate::IndexValuePair, QtPrivate__IndexValuePair)
+QT_IMPL_METATYPE_EXTERN_TAGGED(Qt::Orientation, Qt__Orientation)
+QT_IMPL_METATYPE_EXTERN_TAGGED(QItemSelectionModel::SelectionFlags,
+                               QItemSelectionModel__SelectionFlags)
+
 inline QDebug operator<<(QDebug stream, const RequestedData &data)
 {
     return stream.nospace() << "RequestedData[start=" << data.start << ", end=" << data.end << ", roles=" << data.roles << "]";
@@ -125,6 +99,7 @@ void QAbstractItemModelReplicaImplementation::registerMetatypes()
     qRegisterMetaType<QItemSelectionModel::SelectionFlags>();
     qRegisterMetaType<QSize>();
     qRegisterMetaType<QIntHash>();
+    qRegisterMetaType<QList<int>>();
 }
 
 void QAbstractItemModelReplicaImplementation::initializeModelConnections()
@@ -264,6 +239,8 @@ void QAbstractItemModelReplicaImplementation::onRowsInserted(const QtPrivate::In
     auto parentItem = cacheData(parentIndex);
     q->beginInsertRows(parentIndex, start, end);
     parentItem->insertChildren(start, end);
+    for (int i = start; i <= end; ++i)
+        m_headerData[1].append(CacheEntry());
     q->endInsertRows();
     if (!parentItem->hasChildren && parentItem->columnCount > 0) {
         parentItem->hasChildren = true;
@@ -290,6 +267,8 @@ void QAbstractItemModelReplicaImplementation::onColumnsInserted(const QtPrivate:
             return;
     q->beginInsertColumns(parentIndex, start, end);
     parentItem->columnCount += end - start + 1;
+    for (int i = start; i <= end; ++i)
+        m_headerData[0].append(CacheEntry());
     q->endInsertColumns();
     if (!parentItem->hasChildren && parentItem->children.size() > 0) {
         parentItem->hasChildren = true;
@@ -311,6 +290,7 @@ void QAbstractItemModelReplicaImplementation::onRowsRemoved(const QtPrivate::Ind
     q->beginRemoveRows(parentIndex, start, end);
     if (parentItem)
         parentItem->removeChildren(start, end);
+    m_headerData[1].erase(m_headerData[1].begin() + start, m_headerData[1].begin() + end + 1);
     q->endRemoveRows();
 }
 
@@ -578,7 +558,7 @@ void QAbstractItemModelReplicaImplementation::fetchPendingData()
 
     std::vector<RequestedData> finalRequests;
     RequestedData curData;
-    for (const RequestedData &data : qExchange(m_requestedData, {})) {
+    for (const RequestedData &data : std::exchange(m_requestedData, {})) {
         qCDebug(QT_REMOTEOBJECT_MODELS) << Q_FUNC_INFO << "REQUESTED start=" << data.start << "end=" << data.end << "roles=" << data.roles;
 
         Q_ASSERT(!data.start.isEmpty());
@@ -669,7 +649,7 @@ void QAbstractItemModelReplicaImplementation::onHeaderDataChanged(Qt::Orientatio
     // TODO clean cache
     const int index = orientation == Qt::Horizontal ? 0 : 1;
     QList<CacheEntry> &entries = m_headerData[index];
-    for (int i = first; i < last; ++i )
+    for (int i = first; i <= last && i < entries.size(); ++i )
         entries[i].data.clear();
     emit q->headerDataChanged(orientation, first, last);
 }
@@ -679,7 +659,7 @@ void QAbstractItemModelReplicaImplementation::fetchPendingHeaderData()
     QList<int> roles;
     QList<int> sections;
     QList<Qt::Orientation> orientations;
-    for (const RequestedHeaderData &data : qAsConst(m_requestedHeaderData)) {
+    for (const RequestedHeaderData &data : std::as_const(m_requestedHeaderData)) {
         roles.push_back(data.role);
         sections.push_back(data.section);
         orientations.push_back(data.orientation);
@@ -695,7 +675,7 @@ void QAbstractItemModelReplicaImplementation::onLayoutChanged(const QtPrivate::I
                                                               QAbstractItemModel::LayoutChangeHint hint)
 {
     QList<QPersistentModelIndex> indexes;
-    for (const QtPrivate::ModelIndex &parent : qAsConst(parents)) {
+    for (const QtPrivate::ModelIndex &parent : std::as_const(parents)) {
         const QModelIndex parentIndex = toQModelIndex(QtPrivate::IndexList{parent}, q);
         indexes << QPersistentModelIndex(parentIndex);
     }
