@@ -1,5 +1,6 @@
 // Copyright (C) 2017 Ford Motor Company
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:data-parser
 
 #include "qremoteobjectsource.h"
 #include "qremoteobjectsource_p.h"
@@ -170,7 +171,7 @@ QRemoteObjectSourceBase::QRemoteObjectSourceBase(QObject *obj, Private *d, const
 
     setConnections();
 
-    const auto nChildren = api->m_models.count() + api->m_subclasses.count();
+    const auto nChildren = api->m_models.size() + api->m_subclasses.size();
     if (nChildren > 0) {
         QList<int> roles;
         const int numProperties = api->propertyCount();
@@ -315,7 +316,7 @@ void QRemoteObjectSourceBase::resetObject(QObject *newObject)
     if (newObject)
         setConnections();
 
-    const auto nChildren = m_api->m_models.count() + m_api->m_subclasses.count();
+    const auto nChildren = m_api->m_models.size() + m_api->m_subclasses.size();
     if (nChildren == 0)
         return;
 
@@ -355,7 +356,7 @@ QRemoteObjectRootSource::~QRemoteObjectRootSource()
     // removeListener tries to modify d->m_listeners, this is O(N²),
     // so clear d->m_listeners prior to calling unregister (consume loop).
     // We can do this, because we don't care about the return value of removeListener() here.
-    for (QtROIoDeviceBase *io : qExchange(d->m_listeners, {})) {
+    for (QtROIoDeviceBase *io : std::exchange(d->m_listeners, {})) {
         removeListener(io, true);
     }
     delete d;
@@ -466,7 +467,7 @@ void QRemoteObjectSourceBase::handleMetaCall(int index, QMetaObject::Call call,
         propertyIndex = internalIndex;
     }
 
-    qCDebug(QT_REMOTEOBJECT) << "# Listeners" << d->m_listeners.length();
+    qCDebug(QT_REMOTEOBJECT) << "# Listeners" << d->m_listeners.size();
     qCDebug(QT_REMOTEOBJECT) << "Invoke args:" << m_object
                              << (call == 0 ? QLatin1String("InvokeMetaMethod") : QStringLiteral("Non-invoked call: %d").arg(call))
                              << m_api->signalSignature(index) << *marshalArgs(index, a);
@@ -499,7 +500,7 @@ int QRemoteObjectRootSource::removeListener(QtROIoDeviceBase *io, bool shouldSen
         d->codec->serializeRemoveObjectPacket(m_api->name());
         d->codec->send(io);
     }
-    return int(d->m_listeners.length());
+    return int(d->m_listeners.size());
 }
 
 int QRemoteObjectSourceBase::qt_metacall(QMetaObject::Call call, int methodId, void **a)
